@@ -1,3 +1,28 @@
+/* $Id: MainWindow.cc 82 2006-08-03 00:16:56Z lennart $ */
+
+/***
+  This file is part of paman.
+ 
+  paman is free software; you can redistribute it and/or modify
+  it under the terms of the GNU Lesser General Public License as published
+  by the Free Software Foundation; either version 2 of the License,
+  or (at your option) any later version.
+ 
+  paman is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+  General Public License for more details.
+ 
+  You should have received a copy of the GNU Lesser General Public License
+  along with paman; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+  USA.
+***/
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <iostream>
 #include <string>
 #include <sstream>
@@ -198,11 +223,20 @@ void MainWindow::updateInfo(ModuleInfo &i) {
 
 void MainWindow::updateInfo(SinkInputInfo &i) {
     char t[256];
+    
+    SinkInfo *si = serverInfoManager->getSinkInfo(i.sink);
+    if (!si)
+        return;
+    
+    if (i.treeRef) {
+        Gtk::TreePath path = i.treeRef.get_path();
+        path.up();
+        
+        if (path != si->treeRef.get_path())
+            deviceTreeStore->erase(deviceTreeStore->get_iter(i.treeRef.get_path()));
+    }
+
     if (!i.treeRef) {
-        SinkInfo *si = serverInfoManager->getSinkInfo(i.sink);
-        if (!si)
-            return;
-            
         Gtk::TreeIter iter = deviceTreeStore->get_iter(si->treeRef.get_path());
         i.treeRef = Gtk::TreeRowReference(deviceTreeStore, Gtk::TreePath(deviceTreeStore->append(iter->children())));
     }
@@ -220,11 +254,20 @@ void MainWindow::updateInfo(SinkInputInfo &i) {
 
 void MainWindow::updateInfo(SourceOutputInfo &i) {
     char t[256];
-    if (!i.treeRef) {
-        SourceInfo *si = serverInfoManager->getSourceInfo(i.source);
-        if (!si)
-            return;
+    
+    SourceInfo *si = serverInfoManager->getSourceInfo(i.source);
+    if (!si)
+        return;
+
+    if (i.treeRef) {
+        Gtk::TreePath path = i.treeRef.get_path();
+        path.up();
         
+        if (path != si->treeRef.get_path())
+            deviceTreeStore->erase(deviceTreeStore->get_iter(i.treeRef.get_path()));
+    }
+    
+    if (!i.treeRef) {
         Gtk::TreeIter iter = deviceTreeStore->get_iter(si->treeRef.get_path());
         i.treeRef = Gtk::TreeRowReference(deviceTreeStore, Gtk::TreePath(deviceTreeStore->append(iter->children())));
     }
