@@ -1,16 +1,45 @@
+/* $Id: ServerInfoManager.cc 78 2006-06-21 14:10:54Z lennart $ */
+
+/***
+  This file is part of paman.
+ 
+  paman is free software; you can redistribute it and/or modify
+  it under the terms of the GNU Lesser General Public License as published
+  by the Free Software Foundation; either version 2 of the License,
+  or (at your option) any later version.
+ 
+  paman is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+  General Public License for more details.
+ 
+  You should have received a copy of the GNU Lesser General Public License
+  along with paman; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+  USA.
+***/
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <iostream>
-#include <polyp/polyplib-error.h>
+
+#include <pulse/pulseaudio.h>
 
 #include "ServerInfoManager.hh"
 #include "paman.hh"
 
-SinkInfo::SinkInfo(const struct pa_sink_info &i) :
+SinkInfo::SinkInfo(const pa_sink_info &i) :
+    driver(i.driver),
     name(i.name),
     index(i.index),
     sample_spec(i.sample_spec),
+    channel_map(i.channel_map),
     monitor_source(i.monitor_source),
     owner_module(i.owner_module),
     volume(i.volume),
+    hw_volume_supported(!!(i.flags & PA_SINK_HW_VOLUME_CTRL)),
     latency(i.latency),
     monitor_source_name(i.monitor_source_name),
     window(NULL) {
@@ -24,13 +53,16 @@ SinkInfo::~SinkInfo() {
         delete window;
 }
     
-void SinkInfo::update(const struct pa_sink_info &i) {
+void SinkInfo::update(const pa_sink_info &i) {
+    driver = Glib::ustring(i.driver);
     name = Glib::ustring(i.name);
     index = i.index;
     sample_spec = i.sample_spec;
+    channel_map = i.channel_map;
     monitor_source = i.monitor_source;
     owner_module = i.owner_module;
     volume = i.volume;
+    hw_volume_supported = !!(i.flags & PA_SINK_HW_VOLUME_CTRL),
     latency = i.latency;
     monitor_source_name = i.monitor_source_name;
 
@@ -52,12 +84,16 @@ void SinkInfo::showWindow() {
     }
 }
 
-SourceInfo::SourceInfo(const struct pa_source_info &i) :
+SourceInfo::SourceInfo(const pa_source_info &i) :
+    driver(i.driver),
     name(i.name),
     index(i.index),
     sample_spec(i.sample_spec),
+    channel_map(i.channel_map),
     owner_module(i.owner_module),
     monitor_of_sink(i.monitor_of_sink),
+    volume(i.volume),
+    hw_volume_supported(!!(i.flags & PA_SINK_HW_VOLUME_CTRL)),
     latency(i.latency),
     window(NULL) {
 
@@ -71,11 +107,15 @@ SourceInfo::~SourceInfo() {
 }
 
 void SourceInfo::update(const struct pa_source_info &i) {
+    driver = i.driver;
     name = i.name;
     index = i.index;
     sample_spec = i.sample_spec;
+    channel_map = i.channel_map;
     owner_module = i.owner_module;
     monitor_of_sink = i.monitor_of_sink;
+    volume = i.volume;
+    hw_volume_supported = !!(i.flags & PA_SOURCE_HW_VOLUME_CTRL),
     latency = i.latency;
 
     description = i.description ? i.description : "";
@@ -98,8 +138,8 @@ void SourceInfo::showWindow() {
 
 ClientInfo::ClientInfo(const struct pa_client_info &i) :
     index(i.index),
+    driver(i.driver),
     name(i.name),
-    protocol_name(i.protocol_name),
     owner_module(i.owner_module),
     window(NULL) {
 }
@@ -110,8 +150,8 @@ ClientInfo::~ClientInfo() {
 }
 
 void ClientInfo::update(const struct pa_client_info &i) {
+    driver = i.driver;
     name = i.name;
-    protocol_name = i.protocol_name;
     index = i.index;
     owner_module = i.owner_module;
 
@@ -173,9 +213,11 @@ void ModuleInfo::showWindow() {
 }
 
 SinkInputInfo::SinkInputInfo(const struct pa_sink_input_info &i) :
+    driver(i.driver),
     name(i.name),
     index(i.index),
     sample_spec(i.sample_spec),
+    channel_map(i.channel_map),
     sink(i.sink),
     client(i.client),
     owner_module(i.owner_module),
@@ -192,9 +234,11 @@ SinkInputInfo::~SinkInputInfo() {
 }
 
 void SinkInputInfo::update(const struct pa_sink_input_info &i) {
+    driver = i.driver;
     name = i.name;
     index = i.index;
     sample_spec = i.sample_spec;
+    channel_map = i.channel_map;
     sink = i.sink;
     client = i.client;
     owner_module = i.owner_module;
@@ -220,9 +264,11 @@ void SinkInputInfo::showWindow() {
 }
 
 SourceOutputInfo::SourceOutputInfo(const struct pa_source_output_info &i) :
+    driver(i.driver),
     name(i.name),
     index(i.index),
     sample_spec(i.sample_spec),
+    channel_map(i.channel_map),
     source(i.source),
     client(i.client),
     owner_module(i.owner_module),
@@ -238,9 +284,11 @@ SourceOutputInfo::~SourceOutputInfo() {
 }
 
 void SourceOutputInfo::update(const struct pa_source_output_info &i) {
+    driver = i.driver;
     name = i.name;
     index = i.index;
     sample_spec = i.sample_spec;
+    channel_map = i.channel_map;
     source = i.source;
     client = i.client;
     owner_module = i.owner_module;
@@ -268,9 +316,9 @@ SampleInfo::SampleInfo(const struct pa_sample_info &i) :
     index(i.index),
     name(i.name),
     sample_spec(i.sample_spec),
-    volume(i.volume),
     duration(i.duration),
     bytes(i.bytes),
+    volume(i.volume),
     lazy(!!i.lazy),
     filename_valid(false),
     window(NULL) {
@@ -290,9 +338,9 @@ void SampleInfo::update(const struct pa_sample_info &i) {
     index = i.index;
     name = i.name;
     sample_spec = i.sample_spec;
-    volume = i.volume;
     duration = i.duration;
     bytes = i.bytes;
+    volume = i.volume;
     lazy = !!i.lazy;
 
     filename_valid = !!i.filename;
@@ -696,12 +744,25 @@ void ServerInfoManager::removeSampleInfo(uint32_t index) {
     }
 }
 
-void ServerInfoManager::setSinkVolume(uint32_t index, uint32_t volume) {
-    pa_operation_unref(pa_context_set_sink_volume_by_index(&context, index, volume, NULL, NULL));
+/* FIXME: Handle all channels separately. */
+void ServerInfoManager::setSinkVolume(uint32_t index, pa_volume_t volume) {
+    pa_cvolume cvol;
+    pa_cvolume_set(&cvol, sinks[index]->volume.channels, volume);
+    pa_operation_unref(pa_context_set_sink_volume_by_index(&context, index, &cvol, NULL, NULL));
 }
 
-void ServerInfoManager::setSinkInputVolume(uint32_t index, uint32_t volume) {
-    pa_operation_unref(pa_context_set_sink_input_volume(&context, index, volume, NULL, NULL));
+/* FIXME: Handle all channels separately. */
+void ServerInfoManager::setSourceVolume(uint32_t index, pa_volume_t volume) {
+    pa_cvolume cvol;
+    pa_cvolume_set(&cvol, sources[index]->volume.channels, volume);
+    pa_operation_unref(pa_context_set_source_volume_by_index(&context, index, &cvol, NULL, NULL));
+}
+
+/* FIXME: Handle all channels separately. */
+void ServerInfoManager::setSinkInputVolume(uint32_t index, pa_volume_t volume) {
+    pa_cvolume cvol;
+    pa_cvolume_set(&cvol, sinkInputs[index]->volume.channels, volume);
+    pa_operation_unref(pa_context_set_sink_input_volume(&context, index, &cvol, NULL, NULL));
 }
 
 void ServerInfoManager::showStatWindow() {
@@ -729,10 +790,18 @@ void ServerInfoManager::playSample(uint32_t sample, uint32_t sink) {
 
 void ServerInfoManager::runVolumeMeter(const Glib::ustring &source) {
     char t[256];
-    snprintf(t, sizeof(t), "%s %s", PAVUMETER_PATH, source.c_str());
+    snprintf(t, sizeof(t), "%s --record %s", PAVUMETER_PATH, source.c_str());
     Glib::spawn_command_line_async(t);
 }
 
+bool ServerInfoManager::volumeMeterSupported() {
+    gchar *c;
+    c = g_find_program_in_path(PAVUMETER_PATH);
+    g_free(c);
+
+    return !!c;
+}
+
 void ServerInfoManager::killClient(uint32_t index) {
     pa_operation_unref(pa_context_kill_client(&context, index, NULL, NULL));
 }
